@@ -1,6 +1,6 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##      $Id: mhopt.pl,v 2.61 2005/06/06 15:55:02 ehood Exp $
+##      $Id: mhopt.pl,v 2.67 2011/01/09 05:13:14 ehood Exp $
 ##  Author:
 ##      Earl Hood       mhonarc@mhonarc.org
 ##  Description:
@@ -58,6 +58,7 @@ sub get_resources {
 	'datefields=s', # Fields that contains the date of a message
 	'dbfile=s',	# Database/state filename for mhonarc archive
 	'dbfileperms=i',# Octal permission to set DBFILE
+        'debug',        # Turn on debugging
 	'decodeheads',	# Decode all 1522 encoded data in message headers
 	'definevar|definevars=s@',
 			# Define custom resource variables
@@ -67,6 +68,8 @@ sub get_resources {
 	'expiredate=s',	# Message cut-off date
 	'expireage=i',	# Time in seconds from current if message expires
 	'fasttempfiles',# Do not use random filenames for temporary files
+	'followsymlinks',
+                        # Follow/allow symlinks when create files
 	'fileperms=i',	# Octal permission to create files
 	'folrefs',	# Print links to explicit follow-ups/references
 	'footer=s',	# File containing user text for bottom of index page
@@ -124,6 +127,8 @@ sub get_resources {
 	'nodoc',	# Do not print link to doc at end of index page
 	'nofasttempfiles',
 			# Use random filenames for temporary files
+	'nofollowsymlinks',
+			# Do not follow symlinks when creating files
 	'nofolrefs',	# Do not print links to explicit follow-ups/references
 	'nogzipfiles',	# Do not Gzip files
 	'nogziplinks',	# Do not add '.gz' extensions to files
@@ -137,6 +142,8 @@ sub get_resources {
 	'nonews',	# Do not add links to newsgroups
 	'noposixstrftime',
 			# Use own implementation for time format process
+	'noprintxcomments',
+			# Do not print X- comments
 	'noreverse',	# List messages in normal order
 	'nosaveresources',
 			# Do not save resource values in db
@@ -163,6 +170,8 @@ sub get_resources {
 	'perlinc=s@',	# List of paths to search for MIME filters
 	'posixstrftime',
 			# Use POSIX strftime()
+	'printxcomments',
+			# Print X- comments
 	'quiet',	# No status messages while running
 	'rcfile=s@',	# Resource file for mhonarc
 	'reconvert!',	# Reconvert existing messages
@@ -218,6 +227,9 @@ sub get_resources {
     ## Check for help/version options (nothing to do)
     if ($opt{'help'}) 	{ &usage();   return 0; }
     if ($opt{'v'}) 	{ &version(); return 0; }
+    if ($opt{'debug'}) {
+        $DEBUG = 1;
+    }
 
     ## Check std{in,out,err} options
     DUP: {
@@ -303,6 +315,11 @@ sub get_resources {
     require 'mhsingle.pl'  	if $SINGLE;
     require 'mhrmm.pl'  	if $RMM;
     require 'mhnote.pl'  	if $ANNOTATE;
+    if (!$SCAN) {
+	# require readmail library
+	require 'readmail.pl';
+	mhinit_readmail_vars();
+    }
 
     print STDOUT "This is MHonArc v$VERSION, Perl $] $^O\n"  unless $QUIET;
 
@@ -409,6 +426,26 @@ sub get_resources {
 
 	    ## Set %Follow here just incase it does not get recomputed
 	    %Follow = %FollowOld;
+	    
+	    ## Check if %Time is defined
+	    if (!%Time) {
+		my($k,$v);
+		while (($k,$v) = each %IndexNum) {
+		    $Time{$k} = (split(/$X/o, $k, 2))[0];
+		}
+	    }
+	    if ($DoFromAddr && !%FromAddr) {
+		my($k,$v);
+		while (($k,$v) = each %From) {
+		    $FromAddr{$k} = extract_email_address($v);
+		}
+	    }
+	    if ($DoFromName && !%FromName) {
+		my($k,$v);
+		while (($k,$v) = each %From) {
+		    $FromName{$k} = extract_email_name($v);
+		}
+	    }
 	}
 	if (!$IDXONLY) {
 	    if ($#ARGV < 0) { $ADDSINGLE = 1; }	# See if adding single mesg
@@ -426,15 +463,6 @@ sub get_resources {
     ## Clear thread flag if genidx, must be explicitly set
     $THREAD = 0  if $IDXONLY;
 
-    ## Set mail parsing variables.
-    if (!$SCAN) {
-	# require readmail library
-	require 'readmail.pl';
-	mhinit_readmail_vars();
-	$readmail::FormatHeaderFunc = \&mhonarc::htmlize_header;
-	$MHeadCnvFunc = \&readmail::MAILdecode_1522_str;
-    }
-
     ##	Read resource file(s)
     ##	Look for resource in outdir unless existing according to
     ##  current value.
@@ -605,10 +633,15 @@ sub get_resources {
     $KeepOnRmm	= 1  if $opt{'keeponrmm'};
     $KeepOnRmm	= 0  if $opt{'nokeeponrmm'};
 
+    $PrintXComments = 1  if $opt{'printxcomments'};
+    $PrintXComments = 0  if $opt{'noprintxcomments'};
+
     $CheckNoArchive = 1  if $opt{'checknoarchive'};
     $CheckNoArchive = 0  if $opt{'nochecknoarchive'};
     $FastTempFiles  = 1  if $opt{'fasttempfiles'};
     $FastTempFiles  = 0  if $opt{'nofasttempfiles'};
+    $FollowSymlinks = 1  if $opt{'followsymlinks'};
+    $FollowSymlinks = 0  if $opt{'nofollowsymlinks'};
     $POSIXstrftime  = 1  if $opt{'posixstrftime'};
     $POSIXstrftime  = 0  if $opt{'noposixstrftime'};
 
@@ -682,6 +715,10 @@ sub get_resources {
 	$UPDATE_ALL = 0;
     }
 
+    ## Clean up list-based resources
+    @ExtraHFields = remove_dups(\@ExtraHFields);
+    @FieldOrder   = remove_dups(\@FieldOrder);
+
     ## Set date names
     &set_date_names(\@weekdays, \@Weekdays, \@months, \@Months);
 
@@ -858,380 +895,371 @@ sub update_data_2_4_to_later {
 ##	Initialize readmail.pl variables
 ##
 sub mhinit_readmail_vars {
+    $readmail::DEBUG = $DEBUG;
+
     ##	Default decoders
-    unless (%readmail::MIMEDecoders) {
-	%readmail::MIMEDecoders = (
-	    '7bit'	       => 'as-is',
-	    '8bit'             => 'as-is',
-	    'binary'           => 'as-is',
-	    'base64'           => 'base64::b64decode',
-	    'quoted-printable' => 'quoted_printable::qprdecode',
-	    'x-uuencode'       => 'base64::uudecode',
-	    'x-uue'            => 'base64::uudecode',
-	    'uuencode'         => 'base64::uudecode',
-	);
-	%readmail::MIMEDecodersSrc = (
-	    'base64'           => 'base64.pl',
-	    'quoted-printable' => 'qprint.pl',
-	    'x-uuencode'       => 'base64.pl',
-	    'x-uue'            => 'base64.pl',
-	    'uuencode'         => 'base64.pl',
-	);
-	$IsDefault{'MIMEDECODERS'} = 1;
-    }
+    %readmail::MIMEDecoders = (
+	'7bit'             => 'as-is',
+	'8bit'             => 'as-is',
+	'binary'           => 'as-is',
+	'base64'           => 'base64::b64decode',
+	'quoted-printable' => 'quoted_printable::qprdecode',
+	'x-uuencode'       => 'base64::uudecode',
+	'x-uue'            => 'base64::uudecode',
+	'uuencode'         => 'base64::uudecode',
+    );
+    %readmail::MIMEDecodersSrc = (
+	'base64'           => 'base64.pl',
+	'quoted-printable' => 'qprint.pl',
+	'x-uuencode'       => 'base64.pl',
+	'x-uue'            => 'base64.pl',
+	'uuencode'         => 'base64.pl',
+    );
+    $IsDefault{'MIMEDECODERS'} = 1;
 
     ##	Default filters
-    unless (%readmail::MIMEFilters) {
-	%readmail::MIMEFilters = (
-	    # Content-type			Filter
-	    #-----------------------------------------------------------------
-	    "application/ms-tnef",		"m2h_null::filter",
-	    "application/octet-stream",		"m2h_external::filter",
-	    "application/x-patch",		"m2h_text_plain::filter",
-	    "message/delivery-status",  	"m2h_text_plain::filter",
-	    "message/external-body",		"m2h_msg_extbody::filter",
-	    "message/partial",   		"m2h_text_plain::filter",
-	    "text/enriched",    		"m2h_text_enriched::filter",
-	    "text/html",			"m2h_text_html::filter",
-	    "text/plain",			"m2h_text_plain::filter",
-	    "text/richtext",    		"m2h_text_enriched::filter",
-	    "text/tab-separated-values",	"m2h_text_tsv::filter",
-	    "text/x-html",			"m2h_text_html::filter",
+    %readmail::MIMEFilters = (
+	# Content-type			Filter
+	#-----------------------------------------------------------------
+	"application/ms-tnef",		"m2h_null::filter",
+	"application/octet-stream",	"m2h_external::filter",
+	"application/x-patch",		"m2h_text_plain::filter",
+	"message/delivery-status",  	"m2h_text_plain::filter",
+	"message/external-body",	"m2h_msg_extbody::filter",
+	"message/partial",   		"m2h_text_plain::filter",
+	"text/enriched",    		"m2h_text_enriched::filter",
+	"text/html",			"m2h_text_html::filter",
+	"text/plain",			"m2h_text_plain::filter",
+	"text/richtext",    		"m2h_text_enriched::filter",
+	"text/tab-separated-values",	"m2h_text_tsv::filter",
+	"text/x-html",			"m2h_text_html::filter",
 
-	    "application/*",			"m2h_external::filter",
-	    "audio/*",				"m2h_external::filter",
-	    "chemical/*",  			"m2h_external::filter",
-	    "image/*",  			"m2h_external::filter",
-	    "model/*",  			"m2h_external::filter",
-	    "text/*",   			"m2h_text_plain::filter",
-	    "video/*",  			"m2h_external::filter",
+	"application/*",		"m2h_external::filter",
+	"audio/*",			"m2h_external::filter",
+	"chemical/*",  			"m2h_external::filter",
+	"image/*",  			"m2h_external::filter",
+	"model/*",  			"m2h_external::filter",
+	"text/*",   			"m2h_text_plain::filter",
+	"video/*",  			"m2h_external::filter",
 
-	    "x-sun-attachment",			"m2h_text_plain::filter",
-	);
+	"x-sun-attachment",		"m2h_text_plain::filter",
+    );
 
-	%readmail::MIMEFiltersSrc = (
-	    # Content-type			Filter
-	    #-----------------------------------------------------------------
-	    "application/ms-tnef",		"mhnull.pl",
-	    "application/octet-stream",		"mhexternal.pl",
-	    "application/x-patch",		"mhtxtplain.pl",
-	    "message/delivery-status",  	"mhtxtplain.pl",
-	    "message/external-body",		"mhmsgextbody.pl",
-	    "message/partial",   		"mhtxtplain.pl",
-	    "text/enriched",    		"mhtxtenrich.pl",
-	    "text/html",			"mhtxthtml.pl",
-	    "text/plain",			"mhtxtplain.pl",
-	    "text/richtext",    		"mhtxtenrich.pl",
-	    "text/tab-separated-values",	"mhtxttsv.pl",
-	    "text/x-html",			"mhtxthtml.pl",
+    %readmail::MIMEFiltersSrc = (
+	# Content-type			Filter
+	#-----------------------------------------------------------------
+	"application/ms-tnef",		"mhnull.pl",
+	"application/octet-stream",	"mhexternal.pl",
+	"application/x-patch",		"mhtxtplain.pl",
+	"message/delivery-status",  	"mhtxtplain.pl",
+	"message/external-body",	"mhmsgextbody.pl",
+	"message/partial",   		"mhtxtplain.pl",
+	"text/enriched",    		"mhtxtenrich.pl",
+	"text/html",			"mhtxthtml.pl",
+	"text/plain",			"mhtxtplain.pl",
+	"text/richtext",    		"mhtxtenrich.pl",
+	"text/tab-separated-values",	"mhtxttsv.pl",
+	"text/x-html",			"mhtxthtml.pl",
 
-	    "application/*",			"mhexternal.pl",
-	    "audio/*",				"mhexternal.pl",
-	    "chemical/*",  			"mhexternal.pl",
-	    "image/*",  			"mhexternal.pl",
-	    "model/*",  			"mhexternal.pl",
-	    "text/*",   			"mhtxtplain.pl",
-	    "video/*",  			"mhexternal.pl",
+	"application/*",		"mhexternal.pl",
+	"audio/*",			"mhexternal.pl",
+	"chemical/*",  			"mhexternal.pl",
+	"image/*",  			"mhexternal.pl",
+	"model/*",  			"mhexternal.pl",
+	"text/*",   			"mhtxtplain.pl",
+	"video/*",  			"mhexternal.pl",
 
-	    "x-sun-attachment",			"mhtxtplain.pl",
-	);
-	$IsDefault{'MIMEFILTERS'} = 1;
-    }
+	"x-sun-attachment",		"mhtxtplain.pl",
+    );
+    $IsDefault{'MIMEFILTERS'} = 1;
 
     ##  Default filter arguments
-    unless (%readmail::MIMEFiltersArgs) {
-	%readmail::MIMEFiltersArgs = (
-	    # Content-type			Arguments
-	    #-----------------------------------------------------------------
-	    'm2h_external::filter',		'inline',
-	);
-	$IsDefault{'MIMEARGS'} = 1;
-    }
+    %readmail::MIMEFiltersArgs = (
+	# Content-type			Arguments
+	#-----------------------------------------------------------------
+	'm2h_external::filter',		'inline',
+    );
+    $IsDefault{'MIMEARGS'} = 1;
 
     ##  Charset filters
-    unless (%readmail::MIMECharSetConverters) {
-	%readmail::MIMECharSetConverters = (
-	    # Character set		Converter Function
-	    #-----------------------------------------------------------------
-	    'plain',     		'mhonarc::htmlize',
-	    'us-ascii',   		'mhonarc::htmlize',
-	    'iso-8859-1',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-2',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-3',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-4',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-5',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-6',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-7',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-8',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-9',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-10',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-11',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-13',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-14',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-15',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-8859-16',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-2022-jp',   		'MHonArc::CharEnt::str2sgml',
-	    'iso-2022-kr',    		'MHonArc::CharEnt::str2sgml',
-	    'euc-jp',    		'MHonArc::CharEnt::str2sgml',
-	    'utf-8',    		'MHonArc::CharEnt::str2sgml',
-	    'cp866',    		'MHonArc::CharEnt::str2sgml',
-	    'cp932',    		'MHonArc::CharEnt::str2sgml',
-	    'cp936',    		'MHonArc::CharEnt::str2sgml',
-	    'cp949',    		'MHonArc::CharEnt::str2sgml',
-	    'cp950',    		'MHonArc::CharEnt::str2sgml',
-	    'cp1250',   		'MHonArc::CharEnt::str2sgml',
-	    'cp1251',   		'MHonArc::CharEnt::str2sgml',
-	    'cp1252',   		'MHonArc::CharEnt::str2sgml',
-	    'cp1253',   		'MHonArc::CharEnt::str2sgml',
-	    'cp1254',   		'MHonArc::CharEnt::str2sgml',
-	    'cp1255',   		'MHonArc::CharEnt::str2sgml',
-	    'cp1256',   		'MHonArc::CharEnt::str2sgml',
-	    'cp1257',   		'MHonArc::CharEnt::str2sgml',
-	    'cp1258',   		'MHonArc::CharEnt::str2sgml',
-	    'koi-0',            	'MHonArc::CharEnt::str2sgml',
-	    'koi-7',            	'MHonArc::CharEnt::str2sgml',
-	    'koi8-a',            	'MHonArc::CharEnt::str2sgml',
-	    'koi8-b',            	'MHonArc::CharEnt::str2sgml',
-	    'koi8-e',            	'MHonArc::CharEnt::str2sgml',
-	    'koi8-f',            	'MHonArc::CharEnt::str2sgml',
-	    'koi8-r',            	'MHonArc::CharEnt::str2sgml',
-	    'koi8-u',            	'MHonArc::CharEnt::str2sgml',
-	    'gost19768-87',            	'MHonArc::CharEnt::str2sgml',
-	    'viscii',            	'MHonArc::CharEnt::str2sgml',
-	    'big5-eten',		'MHonArc::CharEnt::str2sgml',
-	    'big5-hkscs',		'MHonArc::CharEnt::str2sgml',
-	    'gb2312',    		'MHonArc::CharEnt::str2sgml',
-	    'macarabic',		'MHonArc::CharEnt::str2sgml',
-	    'maccentraleurroman',	'MHonArc::CharEnt::str2sgml',
-	    'maccroatian',		'MHonArc::CharEnt::str2sgml',
-	    'maccyrillic',		'MHonArc::CharEnt::str2sgml',
-	    'macgreek',		        'MHonArc::CharEnt::str2sgml',
-	    'machebrew',		'MHonArc::CharEnt::str2sgml',
-	    'macicelandic',		'MHonArc::CharEnt::str2sgml',
-	    'macromanian',		'MHonArc::CharEnt::str2sgml',
-	    'macroman',		        'MHonArc::CharEnt::str2sgml',
-	    'macthai',		        'MHonArc::CharEnt::str2sgml',
-	    'macturkish',		'MHonArc::CharEnt::str2sgml',
-	    'hp-roman8',		'MHonArc::CharEnt::str2sgml',
-	    'default',     		'-ignore-',
-	);
-	%readmail::MIMECharSetConvertersSrc = (
-	    # Character set		Converter Function
-	    #-----------------------------------------------------------------
-	    'plain',     		undef,
-	    'us-ascii',   		undef,
-	    'iso-8859-1',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-2',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-3',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-4',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-5',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-6',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-7',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-8',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-9',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-10',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-11',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-13',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-14',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-15',   		'MHonArc/CharEnt.pm',
-	    'iso-8859-16',   		'MHonArc/CharEnt.pm',
-	    'iso-2022-jp',   		'MHonArc/CharEnt.pm',
-	    'iso-2022-kr',    		'MHonArc/CharEnt.pm',
-	    'euc-jp',    		'MHonArc/CharEnt.pm',
-	    'utf-8',    		'MHonArc/CharEnt.pm',
-	    'cp866',    		'MHonArc/CharEnt.pm',
-	    'cp932',    		'MHonArc/CharEnt.pm',
-	    'cp936',    		'MHonArc/CharEnt.pm',
-	    'cp949',    		'MHonArc/CharEnt.pm',
-	    'cp950',    		'MHonArc/CharEnt.pm',
-	    'cp1250',   		'MHonArc/CharEnt.pm',
-	    'cp1251',   		'MHonArc/CharEnt.pm',
-	    'cp1252',   		'MHonArc/CharEnt.pm',
-	    'cp1253',   		'MHonArc/CharEnt.pm',
-	    'cp1254',   		'MHonArc/CharEnt.pm',
-	    'cp1255',   		'MHonArc/CharEnt.pm',
-	    'cp1256',   		'MHonArc/CharEnt.pm',
-	    'cp1257',   		'MHonArc/CharEnt.pm',
-	    'cp1258',   		'MHonArc/CharEnt.pm',
-	    'koi-0',            	'MHonArc/CharEnt.pm',
-	    'koi-7',            	'MHonArc/CharEnt.pm',
-	    'koi8-a',            	'MHonArc/CharEnt.pm',
-	    'koi8-b',            	'MHonArc/CharEnt.pm',
-	    'koi8-e',            	'MHonArc/CharEnt.pm',
-	    'koi8-f',            	'MHonArc/CharEnt.pm',
-	    'koi8-r',            	'MHonArc/CharEnt.pm',
-	    'koi8-u',            	'MHonArc/CharEnt.pm',
-	    'gost19768-87',            	'MHonArc/CharEnt.pm',
-	    'viscii',            	'MHonArc/CharEnt.pm',
-	    'big5-eten',		'MHonArc/CharEnt.pm',
-	    'big5-hkscs',		'MHonArc/CharEnt.pm',
-	    'gb2312',    		'MHonArc/CharEnt.pm',
-	    'macarabic',		'MHonArc/CharEnt.pm',
-	    'maccentraleurroman',	'MHonArc/CharEnt.pm',
-	    'maccroatian',		'MHonArc/CharEnt.pm',
-	    'maccyrillic',		'MHonArc/CharEnt.pm',
-	    'macgreek',		        'MHonArc/CharEnt.pm',
-	    'machebrew',		'MHonArc/CharEnt.pm',
-	    'macicelandic',		'MHonArc/CharEnt.pm',
-	    'macromanian',		'MHonArc/CharEnt.pm',
-	    'macroman',		        'MHonArc/CharEnt.pm',
-	    'macthai',		        'MHonArc/CharEnt.pm',
-	    'macturkish',		'MHonArc/CharEnt.pm',
-	    'hp-roman8',		'MHonArc/CharEnt.pm',
-	    'default',     		undef,
-	);
-	$IsDefault{'CHARSETCONVERTERS'} = 1;
-    }
+    %readmail::MIMECharSetConverters = (
+	# Character set		Converter Function
+	#-----------------------------------------------------------------
+	'plain',     		'mhonarc::htmlize',
+	'us-ascii',   		'mhonarc::htmlize',
+	'iso-8859-1',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-2',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-3',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-4',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-5',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-6',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-7',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-8',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-9',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-10',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-11',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-13',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-14',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-15',   	'MHonArc::CharEnt::str2sgml',
+	'iso-8859-16',   	'MHonArc::CharEnt::str2sgml',
+	'iso-2022-jp',   	'MHonArc::CharEnt::str2sgml',
+	'iso-2022-kr',    	'MHonArc::CharEnt::str2sgml',
+	'euc-jp',    		'MHonArc::CharEnt::str2sgml',
+	'utf-8',    		'MHonArc::CharEnt::str2sgml',
+	'cp866',    		'MHonArc::CharEnt::str2sgml',
+	'cp932',    		'MHonArc::CharEnt::str2sgml',
+	'cp936',    		'MHonArc::CharEnt::str2sgml',
+	'cp949',    		'MHonArc::CharEnt::str2sgml',
+	'cp950',    		'MHonArc::CharEnt::str2sgml',
+	'cp1250',   		'MHonArc::CharEnt::str2sgml',
+	'cp1251',   		'MHonArc::CharEnt::str2sgml',
+	'cp1252',   		'MHonArc::CharEnt::str2sgml',
+	'cp1253',   		'MHonArc::CharEnt::str2sgml',
+	'cp1254',   		'MHonArc::CharEnt::str2sgml',
+	'cp1255',   		'MHonArc::CharEnt::str2sgml',
+	'cp1256',   		'MHonArc::CharEnt::str2sgml',
+	'cp1257',   		'MHonArc::CharEnt::str2sgml',
+	'cp1258',   		'MHonArc::CharEnt::str2sgml',
+	'koi-0',            	'MHonArc::CharEnt::str2sgml',
+	'koi-7',            	'MHonArc::CharEnt::str2sgml',
+	'koi8-a',            	'MHonArc::CharEnt::str2sgml',
+	'koi8-b',            	'MHonArc::CharEnt::str2sgml',
+	'koi8-e',            	'MHonArc::CharEnt::str2sgml',
+	'koi8-f',            	'MHonArc::CharEnt::str2sgml',
+	'koi8-r',            	'MHonArc::CharEnt::str2sgml',
+	'koi8-u',            	'MHonArc::CharEnt::str2sgml',
+	'gost19768-87',         'MHonArc::CharEnt::str2sgml',
+	'viscii',            	'MHonArc::CharEnt::str2sgml',
+	'big5-eten',		'MHonArc::CharEnt::str2sgml',
+	'big5-hkscs',		'MHonArc::CharEnt::str2sgml',
+	'gb2312',    		'MHonArc::CharEnt::str2sgml',
+	'macarabic',		'MHonArc::CharEnt::str2sgml',
+	'maccentraleurroman',	'MHonArc::CharEnt::str2sgml',
+	'maccroatian',		'MHonArc::CharEnt::str2sgml',
+	'maccyrillic',		'MHonArc::CharEnt::str2sgml',
+	'macgreek',		'MHonArc::CharEnt::str2sgml',
+	'machebrew',		'MHonArc::CharEnt::str2sgml',
+	'macicelandic',		'MHonArc::CharEnt::str2sgml',
+	'macromanian',		'MHonArc::CharEnt::str2sgml',
+	'macroman',		'MHonArc::CharEnt::str2sgml',
+	'macthai',		'MHonArc::CharEnt::str2sgml',
+	'macturkish',		'MHonArc::CharEnt::str2sgml',
+	'hp-roman8',		'MHonArc::CharEnt::str2sgml',
+	'default',     		'-ignore-',
+    );
+    %readmail::MIMECharSetConvertersSrc = (
+	# Character set		Converter Function
+	#-----------------------------------------------------------------
+	'plain',     		undef,
+	'us-ascii',   		undef,
+	'iso-8859-1',   	'MHonArc/CharEnt.pm',
+	'iso-8859-2',   	'MHonArc/CharEnt.pm',
+	'iso-8859-3',   	'MHonArc/CharEnt.pm',
+	'iso-8859-4',   	'MHonArc/CharEnt.pm',
+	'iso-8859-5',   	'MHonArc/CharEnt.pm',
+	'iso-8859-6',   	'MHonArc/CharEnt.pm',
+	'iso-8859-7',   	'MHonArc/CharEnt.pm',
+	'iso-8859-8',   	'MHonArc/CharEnt.pm',
+	'iso-8859-9',   	'MHonArc/CharEnt.pm',
+	'iso-8859-10',   	'MHonArc/CharEnt.pm',
+	'iso-8859-11',   	'MHonArc/CharEnt.pm',
+	'iso-8859-13',   	'MHonArc/CharEnt.pm',
+	'iso-8859-14',   	'MHonArc/CharEnt.pm',
+	'iso-8859-15',   	'MHonArc/CharEnt.pm',
+	'iso-8859-16',   	'MHonArc/CharEnt.pm',
+	'iso-2022-jp',   	'MHonArc/CharEnt.pm',
+	'iso-2022-kr',    	'MHonArc/CharEnt.pm',
+	'euc-jp',    		'MHonArc/CharEnt.pm',
+	'utf-8',    		'MHonArc/CharEnt.pm',
+	'cp866',    		'MHonArc/CharEnt.pm',
+	'cp932',    		'MHonArc/CharEnt.pm',
+	'cp936',    		'MHonArc/CharEnt.pm',
+	'cp949',    		'MHonArc/CharEnt.pm',
+	'cp950',    		'MHonArc/CharEnt.pm',
+	'cp1250',   		'MHonArc/CharEnt.pm',
+	'cp1251',   		'MHonArc/CharEnt.pm',
+	'cp1252',   		'MHonArc/CharEnt.pm',
+	'cp1253',   		'MHonArc/CharEnt.pm',
+	'cp1254',   		'MHonArc/CharEnt.pm',
+	'cp1255',   		'MHonArc/CharEnt.pm',
+	'cp1256',   		'MHonArc/CharEnt.pm',
+	'cp1257',   		'MHonArc/CharEnt.pm',
+	'cp1258',   		'MHonArc/CharEnt.pm',
+	'koi-0',            	'MHonArc/CharEnt.pm',
+	'koi-7',            	'MHonArc/CharEnt.pm',
+	'koi8-a',            	'MHonArc/CharEnt.pm',
+	'koi8-b',            	'MHonArc/CharEnt.pm',
+	'koi8-e',            	'MHonArc/CharEnt.pm',
+	'koi8-f',            	'MHonArc/CharEnt.pm',
+	'koi8-r',            	'MHonArc/CharEnt.pm',
+	'koi8-u',            	'MHonArc/CharEnt.pm',
+	'gost19768-87',         'MHonArc/CharEnt.pm',
+	'viscii',            	'MHonArc/CharEnt.pm',
+	'big5-eten',		'MHonArc/CharEnt.pm',
+	'big5-hkscs',		'MHonArc/CharEnt.pm',
+	'gb2312',    		'MHonArc/CharEnt.pm',
+	'macarabic',		'MHonArc/CharEnt.pm',
+	'maccentraleurroman',	'MHonArc/CharEnt.pm',
+	'maccroatian',		'MHonArc/CharEnt.pm',
+	'maccyrillic',		'MHonArc/CharEnt.pm',
+	'macgreek',		'MHonArc/CharEnt.pm',
+	'machebrew',		'MHonArc/CharEnt.pm',
+	'macicelandic',		'MHonArc/CharEnt.pm',
+	'macromanian',		'MHonArc/CharEnt.pm',
+	'macroman',		'MHonArc/CharEnt.pm',
+	'macthai',		'MHonArc/CharEnt.pm',
+	'macturkish',		'MHonArc/CharEnt.pm',
+	'hp-roman8',		'MHonArc/CharEnt.pm',
+	'default',     		undef,
+    );
+    $IsDefault{'CHARSETCONVERTERS'} = 1;
 
     ##	Default charset aliases
-    unless (%readmail::MIMECharsetAliases) {
-	readmail::MAILset_charset_aliases({ 
-	    'us-ascii'     => [ 'ascii',
-				'ansi_x3.4-1968',
-				'iso646', 'iso646-us', 'iso646.irv:1991',
-				'cp367', 'ibm367',
-				'csascii',
-				'iso-ir-6',
-				'us' ],
-	    'iso-8859-1'   => [ 'latin1', 'l1',
-				'iso_8859_1', 'iso_8859-1:1987',
-			        'iso8859-1', 'iso8859_1', '8859-1', '8859_1',
-				'cp819', 'ibm819',
-				'x-mac-latin1',
-				'iso-ir-100' ],
-	    'iso-8859-2'   => [ 'latin2', 'l2',
-				'iso_8859_2', 'iso_8859-2:1987',
-			        'iso8859-2', 'iso8859_2', '8859-2', '8859_2',
-				'iso-ir-101' ],
-	    'iso-8859-3'   => [ 'latin3', 'l3',
-				'iso_8859_3', 'iso_8859-3:1988',
-			        'iso8859-3', 'iso8859_3', '8859-3', '8859_3',
-				'iso-ir-109' ],
-	    'iso-8859-4'   => [ 'latin4', 'l4',
-				'iso_8859_4', 'iso_8859-4:1988',
-			        'iso8859-4', 'iso8859_4', '8859-4', '8859_4',
-				'iso-ir-110' ],
-	    'iso-8859-5'   => [ 'iso_8859-5:1988',
-				'cyrillic',
-				'iso-ir-144' ],
-	    'iso-8859-6'   => [ 'iso_8859-6:1987',
-				'arabic',
-				'asmo-708',
-				'ecma-114',
-				'iso-ir-127' ],
-	    'iso-8859-7'   => [ 'iso_8859-7:1987',
-				'greek', 'greek8',
-				'ecma-118',
-				'elot_928',
-				'iso-ir-126' ],
-	    'iso-8859-8'   => [ 'iso-8859-8-i', 'iso_8859-8:1988',
-				'hebrew',
-				'iso-ir-138' ],
-	    'iso-8859-9'   => [ 'latin5', 'l5',
-				'iso_8859_9', 'iso-8859_9:1989',
-			        'iso8859-9', 'iso8859_9', '8859-9', '8859_9',
-				'iso-ir-148' ],
-	    'iso-8859-10'  => [ 'latin6', 'l6',
-				'iso_8859_10', 'iso_8859-10:1993',
-			        'iso8859-10', 'iso8859_10',
-				'8859-10', '8859_10',
-				'iso-ir-157' ],
-	    'iso-8859-13'  => [ 'latin7' ,'l7' ],
-	    'iso-8859-14'  => [ 'latin8' ,'l8' ],
-	    'iso-8859-15'  => [ 'latin9', 'latin0', 'l9', 'l0',
-				'iso_8859_15',
-				'iso8859-15', 'iso8859_15',
-				'8859-15', '8859_15' ],
-	    'iso-2022-jp'  => [ 'iso-2022-jp-1' ],
-	    'utf-8'        => [ 'utf8' ],
-	    'cp932'        => [ 'shiftjis', 'shift_jis', 'shift-jis',
-				'x-sjis',
-				'ms_kanji',
-				'csshiftjis' ],
-	    'cp936'        => [ 'gbk',
-				'ms936',
-				'windows-936' ],
-	    'cp949'        => [ 'euc-kr',
-				'ks_c_5601-1987', 'ks_c_5601-1989',
-				'ksc_5601',
-				'iso-ir-149',
-				'windows-949', 'ms949',
-				'korean' ],
-	    'cp950'        => [ 'windows-950' ],
-	    'cp1250'       => [ 'windows-1250' ],
-	    'cp1251'       => [ 'windows-1251' ],
-	    'cp1252'       => [ 'windows-1252' ],
-	    'cp1253'       => [ 'windows-1253' ],
-	    'cp1254'       => [ 'windows-1254' ],
-	    'cp1255'       => [ 'windows-1255' ],
-	    'cp1256'       => [ 'windows-1256' ],
-	    'cp1257'       => [ 'windows-1257' ],
-	    'cp1258'       => [ 'windows-1258' ],
-	    'koi-0'          => [ 'gost-13052' ],
-	    'koi8-e'         => [ 'iso-ir-111',
-				  'ecma-113:1986' ],
-	    'koi8-r'         => [ 'cp878' ],
-	    'gost-19768-87'  => [ 'ecma-cyrillic',
-				  'ecma-113', 'ecma-113:1988' ],
-	    'big5-eten'      => [ 'big5', 'csbig5',
-				  'tcs-big5', 'tcsbig5' ],
-	    'big5-hkscs'     => [ 'big5hkscs', 'big5hk',
-				  'hkscs-big5', 'hk-big5' ],
-	    'gb2312'	     => [ 'gb_2312-80', 'csgb2312', 'hz-gb-2312',
-				  'iso-ir-58',
-				  'euc-cn',
-				  'chinese',
-				  'csiso58gb231280' ],
-	    'macarabic'	         => [ 'apple-arabic',
-				      'x-mac-arabic' ],
-	    'maccentraleurroman' => [ 'apple-centeuro',
-				      'x-mac-centraleurroman' ],
-	    'maccroatian'        => [ 'apple-croatian',
-				      'x-mac-croatian' ],
-	    'maccyrillic'        => [ 'apple-cyrillic',
-				      'x-mac-cyrillic' ],
-	    'macgreek'	         => [ 'apple-greek',
-				      'x-mac-greek' ],
-	    'machebrew'	         => [ 'apple-hebrew',
-				      'x-mac-hebrew' ],
-	    'macicelandic'       => [ 'apple-iceland',
-				      'x-mac-icelandic' ],
-	    'macromanian'        => [ 'apple-romanian',
-				      'x-mac-romanian' ],
-	    'macroman'	         => [ 'apple-roman',
-				      'mac', 'macintosh',
-				      'x-mac-roman' ],
-	    'macthai'	         => [ 'apple-thai',
-				      'x-mac-thai' ],
-	    'macturkish'         => [ 'apple-turkish',
-				      'x-mac-turkish' ],
-	});
-	$IsDefault{'CHARSETALIASES'} = 1;
-    }
+    readmail::MAILset_charset_aliases({ 
+	'us-ascii'     => [ 'ascii',
+			    'ansi_x3.4-1968',
+			    'iso646', 'iso646-us', 'iso646.irv:1991',
+			    'cp367', 'ibm367',
+			    'csascii',
+			    'iso-ir-6',
+			    'us' ],
+	'iso-8859-1'   => [ 'latin1', 'l1',
+			    'iso_8859_1', 'iso_8859-1:1987',
+			    'iso8859-1', 'iso8859_1', '8859-1', '8859_1',
+			    'cp819', 'ibm819',
+			    'x-mac-latin1',
+			    'iso-ir-100' ],
+	'iso-8859-2'   => [ 'latin2', 'l2',
+			    'iso_8859_2', 'iso_8859-2:1987',
+			    'iso8859-2', 'iso8859_2', '8859-2', '8859_2',
+			    'iso-ir-101' ],
+	'iso-8859-3'   => [ 'latin3', 'l3',
+			    'iso_8859_3', 'iso_8859-3:1988',
+			    'iso8859-3', 'iso8859_3', '8859-3', '8859_3',
+			    'iso-ir-109' ],
+	'iso-8859-4'   => [ 'latin4', 'l4',
+			    'iso_8859_4', 'iso_8859-4:1988',
+			    'iso8859-4', 'iso8859_4', '8859-4', '8859_4',
+			    'iso-ir-110' ],
+	'iso-8859-5'   => [ 'iso_8859-5:1988',
+			    'cyrillic',
+			    'iso-ir-144' ],
+	'iso-8859-6'   => [ 'iso_8859-6:1987',
+			    'arabic',
+			    'asmo-708',
+			    'ecma-114',
+			    'iso-ir-127' ],
+	'iso-8859-7'   => [ 'iso_8859-7:1987',
+			    'greek', 'greek8',
+			    'ecma-118',
+			    'elot_928',
+			    'iso-ir-126' ],
+	'iso-8859-8'   => [ 'iso-8859-8-i', 'iso_8859-8:1988',
+			    'hebrew',
+			    'iso-ir-138' ],
+	'iso-8859-9'   => [ 'latin5', 'l5',
+			    'iso_8859_9', 'iso-8859_9:1989',
+			    'iso8859-9', 'iso8859_9', '8859-9', '8859_9',
+			    'iso-ir-148' ],
+	'iso-8859-10'  => [ 'latin6', 'l6',
+			    'iso_8859_10', 'iso_8859-10:1993',
+			    'iso8859-10', 'iso8859_10',
+			    '8859-10', '8859_10',
+			    'iso-ir-157' ],
+	'iso-8859-13'  => [ 'latin7' ,'l7' ],
+	'iso-8859-14'  => [ 'latin8' ,'l8' ],
+	'iso-8859-15'  => [ 'latin9', 'latin0', 'l9', 'l0',
+			    'iso_8859_15',
+			    'iso8859-15', 'iso8859_15',
+			    '8859-15', '8859_15' ],
+	'iso-2022-jp'  => [ 'iso-2022-jp-1' ],
+	'utf-8'        => [ 'utf8' ],
+	'cp932'        => [ 'shiftjis', 'shift_jis', 'shift-jis',
+			    'x-sjis',
+			    'ms_kanji',
+			    'csshiftjis' ],
+	'cp936'        => [ 'gbk',
+			    'ms936',
+			    'windows-936' ],
+	'cp949'        => [ 'euc-kr',
+			    'ks_c_5601-1987', 'ks_c_5601-1989',
+			    'ksc_5601',
+			    'iso-ir-149',
+			    'windows-949', 'ms949',
+			    'korean' ],
+	'cp950'        => [ 'windows-950' ],
+	'cp1250'       => [ 'windows-1250' ],
+	'cp1251'       => [ 'windows-1251' ],
+	'cp1252'       => [ 'windows-1252' ],
+	'cp1253'       => [ 'windows-1253' ],
+	'cp1254'       => [ 'windows-1254' ],
+	'cp1255'       => [ 'windows-1255' ],
+	'cp1256'       => [ 'windows-1256' ],
+	'cp1257'       => [ 'windows-1257' ],
+	'cp1258'       => [ 'windows-1258' ],
+	'koi-0'          => [ 'gost-13052' ],
+	'koi8-e'         => [ 'iso-ir-111',
+			      'ecma-113:1986' ],
+	'koi8-r'         => [ 'cp878' ],
+	'gost-19768-87'  => [ 'ecma-cyrillic',
+			      'ecma-113', 'ecma-113:1988' ],
+	'big5-eten'      => [ 'big5', 'csbig5',
+			      'tcs-big5', 'tcsbig5' ],
+	'big5-hkscs'     => [ 'big5hkscs', 'big5hk',
+			      'hkscs-big5', 'hk-big5' ],
+	'gb2312'	     => [ 'gb_2312-80', 'csgb2312', 'hz-gb-2312',
+			          'iso-ir-58',
+			          'euc-cn',
+			          'chinese',
+			          'csiso58gb231280' ],
+	'macarabic'	     => [ 'apple-arabic',
+				  'x-mac-arabic' ],
+	'maccentraleurroman' => [ 'apple-centeuro',
+				  'x-mac-centraleurroman' ],
+	'maccroatian'        => [ 'apple-croatian',
+				  'x-mac-croatian' ],
+	'maccyrillic'        => [ 'apple-cyrillic',
+				  'x-mac-cyrillic' ],
+	'macgreek'	     => [ 'apple-greek',
+				  'x-mac-greek' ],
+	'machebrew'	     => [ 'apple-hebrew',
+				  'x-mac-hebrew' ],
+	'macicelandic'       => [ 'apple-iceland',
+				  'x-mac-icelandic' ],
+	'macromanian'        => [ 'apple-romanian',
+				  'x-mac-romanian' ],
+	'macroman'	     => [ 'apple-roman',
+				  'mac', 'macintosh',
+				  'x-mac-roman' ],
+	'macthai'	     => [ 'apple-thai',
+				  'x-mac-thai' ],
+	'macturkish'         => [ 'apple-turkish',
+				  'x-mac-turkish' ],
+    });
+    $IsDefault{'CHARSETALIASES'} = 1;
 
     ##  Content-Types to exclude:
     ##    Nothing is excluded by default.
-    unless (%readmail::MIMEExcs) {
-	$IsDefault{'MIMEEXCS'} = 1;
-    }
+    %readmail::MIMEExcs = ( );
+    $IsDefault{'MIMEEXCS'} = 1;
 
     ##  Content-Types to only include:
     ##    Blank by default: include everything
-    unless (%readmail::MIMEIncs) {
-	$IsDefault{'MIMEIncs'} = 1;
-    }
+    %readmail::MIMEIncs = ( );
+    $IsDefault{'MIMEIncs'} = 1;
 
     ##  Content-type multipart/alternative preferences
     ##    Note: The variable is not a readmail package variable, but it
     ##	    is used to set readmail package properties.
-    unless (@MIMEAltPrefs) {
-	$IsDefault{'MIMEALTPREFS'} = 1;
-    }
+    @MIMEAltPrefs = ( );
+    $IsDefault{'MIMEALTPREFS'} = 1;
 
     ##	Text encoding
-    unless ($readmail::TextEncode) {
-	$IsDefault{'TEXTENCODE'} = 1;
-    }
+    $readmail::TextEncode = undef;
+    $IsDefault{'TEXTENCODE'} = 1;
+
+    $readmail::FormatHeaderFunc = \&mhonarc::htmlize_header;
+    $MHeadCnvFunc = \&readmail::MAILdecode_1522_str;
 }
 
 ##---------------------------------------------------------------------------
